export declare enum AttributeNames {
    PG_VALUES = "db.postgresql.values",
    PG_PLAN = "db.postgresql.plan",
    IDLE_TIMEOUT_MILLIS = "db.postgresql.idle.timeout.millis",
    MAX_CLIENT = "db.postgresql.max.client"
}
//# sourceMappingURL=AttributeNames.d.ts.map